# -*- Python -*-
# Converted by ./convert_mime_type_table.py from:
# /usr/src2/apache_1.2b6/conf/mime.types
#
content_type_map = \
  {
       'css': 'text/css',
        'js': 'text/javascript',
       'log': 'text/plain',
        'ai':  'application/postscript',
       'aif':  'audio/x-aiff',
      'aifc':  'audio/x-aiff',
      'aiff':  'audio/x-aiff',
        'au':  'audio/basic',
       'avi':  'video/x-msvideo',
     'bcpio':  'application/x-bcpio',
       'bin':  'application/octet-stream',
       'cdf':  'application/x-netcdf',
     'class':  'application/octet-stream',
      'cpio':  'application/x-cpio',
       'cpt':  'application/mac-compactpro',
       'csh':  'application/x-csh',
       'dcr':  'application/x-director',
       'dir':  'application/x-director',
       'dms':  'application/octet-stream',
       'doc':  'application/msword',
       'dvi':  'application/x-dvi',
       'dxr':  'application/x-director',
       'eps':  'application/postscript',
       'etx':  'text/x-setext',
       'exe':  'application/octet-stream',
       'gif':  'image/gif',
      'gtar':  'application/x-gtar',
        'gz':  'application/x-gzip',
       'hdf':  'application/x-hdf',
       'hqx':  'application/mac-binhex40',
       'htm':  'text/html',
      'html':  'text/html',
       'ice':  'x-conference/x-cooltalk',
       'ief':  'image/ief',
       'jpe':  'image/jpeg',
      'jpeg':  'image/jpeg',
       'jpg':  'image/jpeg',
       'kar':  'audio/midi',
     'latex':  'application/x-latex',
       'lha':  'application/octet-stream',
       'lzh':  'application/octet-stream',
       'man':  'application/x-troff-man',
        'me':  'application/x-troff-me',
       'mid':  'audio/midi',
      'midi':  'audio/midi',
       'mif':  'application/x-mif',
       'mov':  'video/quicktime',
     'movie':  'video/x-sgi-movie',
       'mp2':  'audio/mpeg',
       'mpe':  'video/mpeg',
      'mpeg':  'video/mpeg',
       'mpg':  'video/mpeg',
      'mpga':  'audio/mpeg',
       'mp3':  'audio/mpeg',
        'ms':  'application/x-troff-ms',
        'nc':  'application/x-netcdf',
       'oda':  'application/oda',
       'pbm':  'image/x-portable-bitmap',
       'pdb':  'chemical/x-pdb',
       'pdf':  'application/pdf',
       'pgm':  'image/x-portable-graymap',
       'png':  'image/png',
       'pnm':  'image/x-portable-anymap',
       'ppm':  'image/x-portable-pixmap',
       'ppt':  'application/powerpoint',
        'ps':  'application/postscript',
        'qt':  'video/quicktime',
        'ra':  'audio/x-realaudio',
       'ram':  'audio/x-pn-realaudio',
       'ras':  'image/x-cmu-raster',
       'rgb':  'image/x-rgb',
      'roff':  'application/x-troff',
       'rpm':  'audio/x-pn-realaudio-plugin',
       'rtf':  'application/rtf',
       'rtx':  'text/richtext',
       'sgm':  'text/x-sgml',
      'sgml':  'text/x-sgml',
        'sh':  'application/x-sh',
      'shar':  'application/x-shar',
       'sit':  'application/x-stuffit',
       'skd':  'application/x-koan',
       'skm':  'application/x-koan',
       'skp':  'application/x-koan',
       'snd':  'audio/basic',
       'src':  'application/x-wais-source',
   'sv4cpio':  'application/x-sv4cpio',
    'sv4crc':  'application/x-sv4crc',
         't':  'application/x-troff',
       'tar':  'application/x-tar',
       'tcl':  'application/x-tcl',
       'tex':  'application/x-tex',
      'texi':  'application/x-texinfo',
   'texinfo':  'application/x-texinfo',
       'tif':  'image/tiff',
      'tiff':  'image/tiff',
        'tr':  'application/x-troff',
       'tsv':  'text/tab-separated-values',
       'txt':  'text/plain',
     'ustar':  'application/x-ustar',
       'vcd':  'application/x-cdlink',
      'vrml':  'x-world/x-vrml',
       'wav':  'audio/x-wav',
       'wrl':  'x-world/x-vrml',
       'xbm':  'image/x-xbitmap',
       'xpm':  'image/x-xpixmap',
       'xwd':  'image/x-xwindowdump',
       'xyz':  'chemical/x-pdb',
       'zip':  'application/zip',
  }
